@@ -4,6 +4,7 @@ import itertools
 import math
 import operator
 import re
+import warnings
 from fractions import Fraction
 from functools import partial
 from typing import Any, Callable, Dict, List, NoReturn, Optional, Set, Union
@@ -25,9 +26,9 @@ from ._canonicalise import (
     get_type,
     make_validator,
     merged,
-    resolve_all_refs,
 )
 from ._encode import JSONType, encode_canonical_json
+from ._resolve import resolve_all_refs
 
 JSON_STRATEGY: st.SearchStrategy[JSONType] = st.recursive(
     st.none()
@@ -388,6 +389,14 @@ STRING_FORMATS = {
 }
 
 
+def _warn_invalid_regex(pattern: str, err: re.error, kw: str = "pattern") -> None:
+    warnings.warn(
+        f"Got {kw}={pattern!r}, but this is not valid syntax for a Python regular "
+        f"expression ({err}) so it will not be handled by the strategy.  See https://"
+        "json-schema.org/understanding-json-schema/reference/regular_expressions.html"
+    )
+
+
 def string_schema(
     custom_formats: Dict[str, st.SearchStrategy[str]], schema: dict
 ) -> st.SearchStrategy[str]:
@@ -402,14 +411,19 @@ def string_schema(
         # See https://json-schema.org/latest/json-schema-validation.html#format
         strategy = known_formats[schema["format"]]
         if "pattern" in schema:
-            # This isn't really supported, but we'll do our best.
-            strategy = strategy.filter(re.compile(schema["pattern"]).search)
+            try:
+                # This isn't really supported, but we'll do our best with a filter.
+                strategy = strategy.filter(re.compile(schema["pattern"]).search)
+            except re.error as err:
+                _warn_invalid_regex(schema["pattern"], err)
+                return st.nothing()
     elif "pattern" in schema:
         try:
             re.compile(schema["pattern"])
             strategy = st.from_regex(schema["pattern"])
-        except re.error:
+        except re.error as err:
             # Patterns that are invalid in Python, or just malformed
+            _warn_invalid_regex(schema["pattern"], err)
             return st.nothing()
     # If we have size bounds but we're generating strings from a regex or pattern,
     # apply a filter to ensure our size bounds are respected.
@@ -450,6 +464,14 @@ def array_schema(
             if contains_additional is not None:
                 additional_items_strat |= _from_schema_(contains_additional)
 
+        # Hypothesis raises InvalidArgument for empty elements and non-None
+        # max_size, because the user has asked for a possibility which will
+        # never happen... but we can work around that here.
+        if additional_items_strat.is_empty:
+            if min_size >= 1:
+                return st.nothing()
+            max_size = 0
+
         if unique:
 
             @st.composite  # type: ignore
@@ -463,6 +485,8 @@ def array_schema(
                 for strat in items_strats:
                     elems.append(draw(strat.filter(not_seen)))
                     seen.add(encode_canonical_json(elems[-1]))
+                if max_size == 0:
+                    return elems
                 extra_items = st.lists(
                     additional_items_strat.filter(not_seen),
                     min_size=min_size,
@@ -473,6 +497,8 @@ def array_schema(
                 return elems + more_elems
 
             strat = compose_lists_with_filter()
+        elif max_size == 0:
+            strat = st.tuples(*items_strats).map(list)
         else:
             strat = st.builds(
                 operator.add,
@@ -490,7 +516,9 @@ def array_schema(
                 # heterogeneous) and hope it works out anyway.
                 contains_strat = contains_strat.filter(make_validator(items).is_valid)
             items_strat |= contains_strat
-
+        elif items_strat.is_empty and min_size == 0 and max_size is not None:
+            # As above, work around a Hypothesis check for unsatisfiable max_size.
+            return st.builds(list)
         strat = st.lists(
             items_strat,
             min_size=min_size,
@@ -524,6 +552,15 @@ def object_schema(
     additional = schema.get("additionalProperties", {})
     additional_allowed = additional != FALSEY
 
+    for key in list(patterns):
+        try:
+            re.compile(key)
+        except re.error as err:
+            _warn_invalid_regex(key, err, "patternProperties entry")
+            if min_size == 0 and not required:
+                return st.builds(dict)
+            return st.nothing()
+
     dependencies = schema.get("dependencies", {})
     dep_names = {k: v for k, v in dependencies.items() if isinstance(v, list)}
     dep_schemas = {k: v for k, v in dependencies.items() if k not in dep_names}
